@@ -113,20 +113,20 @@ ContactReference::ContactReference(QObject *parent)
 
 
 /* ********************************************************************************************* *
- * Implementation of DigitalContactReference
+ * Implementation of DMRContactReference
  * ********************************************************************************************* */
-DigitalContactReference::DigitalContactReference(QObject *parent)
-  : ContactReference(DigitalContact::staticMetaObject, parent)
+DMRContactReference::DMRContactReference(QObject *parent)
+  : ContactReference(DMRContact::staticMetaObject, parent)
 {
   // pass...
 }
 
 
 /* ********************************************************************************************* *
- * Implementation of DigitalContactRefList
+ * Implementation of DMRContactRefList
  * ********************************************************************************************* */
-DigitalContactRefList::DigitalContactRefList(QObject *parent)
-  : ConfigObjectRefList(DigitalContact::staticMetaObject, parent)
+DMRContactRefList::DMRContactRefList(QObject *parent)
+  : ConfigObjectRefList(DMRContact::staticMetaObject, parent)
 {
   // pass...
 }
@@ -149,20 +149,20 @@ ChannelReference::ChannelReference(QObject *parent)
 
 
 /* ********************************************************************************************* *
- * Implementation of DigitalChannelReference
+ * Implementation of DMRChannelReference
  * ********************************************************************************************* */
-DigitalChannelReference::DigitalChannelReference(QObject *parent)
-  : ChannelReference(DigitalChannel::staticMetaObject, parent)
+DMRChannelReference::DMRChannelReference(QObject *parent)
+  : ChannelReference(DMRChannel::staticMetaObject, parent)
 {
   // pass...
 }
 
 
 /* ********************************************************************************************* *
- * Implementation of AnalogChannelReference
+ * Implementation of FMChannelReference
  * ********************************************************************************************* */
-AnalogChannelReference::AnalogChannelReference(QObject *parent)
-  : ChannelReference(AnalogChannel::staticMetaObject, parent)
+FMChannelReference::FMChannelReference(QObject *parent)
+  : ChannelReference(FMChannel::staticMetaObject, parent)
 {
   // pass...
 }
@@ -185,10 +185,10 @@ ChannelRefList::ChannelRefList(QObject *parent)
 
 
 /* ********************************************************************************************* *
- * Implementation of DigitalChannelRefList
+ * Implementation of DMRChannelRefList
  * ********************************************************************************************* */
-DigitalChannelRefList::DigitalChannelRefList(QObject *parent)
-  : ChannelRefList(DigitalChannel::staticMetaObject, parent)
+DMRChannelRefList::DMRChannelRefList(QObject *parent)
+  : ChannelRefList(DMRChannel::staticMetaObject, parent)
 {
   // pass...
 }
@@ -243,7 +243,7 @@ GPSSystemReference::GPSSystemReference(QObject *parent)
 /* ********************************************************************************************* *
  * Implementation of RadioIDReference
  * ********************************************************************************************* */
-RadioIDReference::RadioIDReference(QObject *parent)
+DMRRadioIDReference::DMRRadioIDReference(QObject *parent)
   : ConfigObjectReference(DMRRadioID::staticMetaObject, parent)
 {
   // pass...
