@@ -30,10 +30,11 @@
 #define VFO_SIZE                  0x00000040 // Size of each VFO settings.
 
 #define NUM_CONTACTS              10000      // Total number of contacts
-#define NUM_CONTACT_BANKS         2500       // Number of contact banks
-#define CONTACTS_PER_BANK         4
-#define CONTACT_BANK_0            0x02680000 // First bank of 4 contacts
-#define CONTACT_BANK_SIZE         0x00000190 // Size of 4 contacts
+#define CONTACTS_PER_BLOCK        4
+#define CONTACT_BLOCK_0           0x02680000 // First bank of 4 contacts
+#define CONTACT_BLOCK_SIZE        0x00000190 // Size of 4 contacts
+#define CONTACT_BANK_SIZE         0x00040000 // Size of one contact bank
+#define CONTACTS_PER_BANK         1000       // Number of contacts per bank
 #define CONTACT_INDEX_LIST        0x02600000 // Address of contact index list
 #define CONTACTS_BITMAP           0x02640000 // Address of contact bitmap
 #define CONTACTS_BITMAP_SIZE      0x00000500 // Size of contact bitmap
@@ -473,6 +474,13 @@ D868UVCodeplug::GeneralSettingsElement::updateConfig(Context &ctx) {
 D868UVCodeplug::D868UVCodeplug(QObject *parent)
   : AnytoneCodeplug(parent)
 {
+}
+
+void
+D868UVCodeplug::clear() {
+  while (this->numImages())
+    remImage(0);
+
   addImage("Anytone AT-D868UV Codeplug");
 
   // Channel bitmap
@@ -489,7 +497,7 @@ D868UVCodeplug::D868UVCodeplug(QObject *parent)
   image(0).addElement(SCAN_BITMAP, SCAN_BITMAP_SIZE);
   // Radio IDs bitmaps
   image(0).addElement(RADIOID_BITMAP, RADIOID_BITMAP_SIZE);
-  // Messag bitmaps
+  // Message bitmaps
   image(0).addElement(MESSAGE_BYTEMAP, MESSAGE_BYTEMAP_SIZE);
   // Status messages
   image(0).addElement(STATUSMESSAGE_BITMAP, STATUSMESSAGE_BITMAP_SIZE);
@@ -502,11 +510,6 @@ D868UVCodeplug::D868UVCodeplug(QObject *parent)
   image(0).addElement(TWO_TONE_FUNCTIONS_BITMAP, TWO_TONE_FUNC_BITMAP_SIZE);
 }
 
-void
-D868UVCodeplug::clear() {
-  // NOOP
-}
-
 void
 D868UVCodeplug::allocateUpdated() {
   this->allocateVFOSettings();
@@ -818,12 +821,14 @@ D868UVCodeplug::allocateContacts() {
     if (1 == ((contact_bitmap[i/8]>>(i%8)) & 0x01))
       continue;
     contactCount++;
-    uint32_t addr = CONTACT_BANK_0+(i/CONTACTS_PER_BANK)*CONTACT_BANK_SIZE;
+    uint32_t bank_addr = CONTACT_BLOCK_0 + (i/CONTACTS_PER_BANK)*CONTACT_BANK_SIZE;
+    uint32_t addr = bank_addr + ((i%CONTACTS_PER_BANK)/CONTACTS_PER_BLOCK)*CONTACT_BLOCK_SIZE;
     if (nullptr == data(addr, 0)) {
-      image(0).addElement(addr, CONTACT_BANK_SIZE);
-      memset(data(addr), 0x00, CONTACT_BANK_SIZE);
+      image(0).addElement(addr, CONTACT_BLOCK_SIZE);
+      memset(data(addr), 0x00, CONTACT_BLOCK_SIZE);
     }
   }
+
   if (contactCount) {
     image(0).addElement(CONTACT_INDEX_LIST, align_size(4*contactCount, 16));
     memset(data(CONTACT_INDEX_LIST), 0xff, align_size(4*contactCount, 16));
@@ -836,11 +841,13 @@ bool
 D868UVCodeplug::encodeContacts(const Flags &flags, Context &ctx, const ErrorStack &err) {
   Q_UNUSED(flags); Q_UNUSED(err)
 
-  QVector<DigitalContact*> contacts;
+  QVector<DMRContact*> contacts;
   // Encode contacts and also collect id<->index map
   for (int i=0; i<ctx.config()->contacts()->digitalCount(); i++) {
-    ContactElement con(data(CONTACT_BANK_0+i*CONTACT_SIZE));
-    DigitalContact *contact = ctx.config()->contacts()->digitalContact(i);
+    uint32_t bank_addr = CONTACT_BLOCK_0 + (i/CONTACTS_PER_BANK)*CONTACT_BANK_SIZE;
+    uint32_t addr = bank_addr + (i%CONTACTS_PER_BANK)*CONTACT_SIZE;
+    ContactElement con(data(addr));
+    DMRContact *contact = ctx.config()->contacts()->digitalContact(i);
     if(! con.fromContactObj(contact, ctx))
       return false;
     ((uint32_t *)data(CONTACT_INDEX_LIST))[i] = qToLittleEndian(i);
@@ -848,12 +855,12 @@ D868UVCodeplug::encodeContacts(const Flags &flags, Context &ctx, const ErrorStac
   }
   // encode index map for contacts
   std::sort(contacts.begin(), contacts.end(),
-            [](DigitalContact *a, DigitalContact *b) {
+            [](DMRContact *a, DMRContact *b) {
     return a->number() < b->number();
   });
   for (int i=0; i<contacts.size(); i++) {
     ContactMapElement el(data(CONTACT_ID_MAP + i*CONTACT_ID_ENTRY_SIZE));
-    el.setID(contacts[i]->number(), (DigitalContact::GroupCall==contacts[i]->type()));
+    el.setID(contacts[i]->number(), (DMRContact::GroupCall==contacts[i]->type()));
     el.setIndex(ctx.index(contacts[i]));
   }
   return true;
@@ -870,8 +877,10 @@ D868UVCodeplug::createContacts(Context &ctx, const ErrorStack &err) {
     uint16_t  bit = i%8, byte = i/8;
     if (1 == ((contact_bitmap[byte]>>bit) & 0x01))
       continue;
-    ContactElement con(data(CONTACT_BANK_0+i*CONTACT_SIZE));
-    if (DigitalContact *obj = con.toContactObj(ctx)) {
+    uint32_t bank_addr = CONTACT_BLOCK_0 + (i/CONTACTS_PER_BANK)*CONTACT_BANK_SIZE;
+    uint32_t addr = bank_addr + (i%CONTACTS_PER_BANK)*CONTACT_SIZE;
+    ContactElement con(data(addr));
+    if (DMRContact *obj = con.toContactObj(ctx)) {
       ctx.config()->contacts()->add(obj); ctx.add(obj, i);
     }
   }
@@ -1091,7 +1100,11 @@ D868UVCodeplug::encodeZones(const Flags &flags, Context &ctx, const ErrorStack &
     for (int j=0; j<ctx.config()->zones()->zone(i)->A()->count(); j++) {
       channels[j] = qToLittleEndian(ctx.index(ctx.config()->zones()->zone(i)->A()->get(j)));
     }
+
+    if (! encodeZone(zidx, ctx.config()->zones()->zone(i), false, flags, ctx, err))
+      return false;
     zidx++;
+
     if (! ctx.config()->zones()->zone(i)->B()->count())
       continue;
 
@@ -1105,11 +1118,20 @@ D868UVCodeplug::encodeZones(const Flags &flags, Context &ctx, const ErrorStack &
     for (int j=0; j<ctx.config()->zones()->zone(i)->B()->count(); j++) {
       channels[j] = qToLittleEndian(ctx.index(ctx.config()->zones()->zone(i)->B()->get(j)));
     }
+
+    if (! encodeZone(zidx, ctx.config()->zones()->zone(i), true, flags, ctx, err))
+      return false;
     zidx++;
   }
   return true;
 }
 
+bool
+D868UVCodeplug::encodeZone(int i, Zone *zone, bool isB, const Flags &flags, Context &ctx, const ErrorStack &err) {
+  Q_UNUSED(i); Q_UNUSED(zone); Q_UNUSED(isB); Q_UNUSED(flags); Q_UNUSED(ctx); Q_UNUSED(err)
+  return true;
+}
+
 bool
 D868UVCodeplug::createZones(Context &ctx, const ErrorStack &err) {
   Q_UNUSED(err)
@@ -1135,16 +1157,30 @@ D868UVCodeplug::createZones(Context &ctx, const ErrorStack &err) {
     // If enabled, create zone with name
     if (! extend_last_zone) {
       last_zone = new Zone(zonename);
+      if (! decodeZone(i, last_zone, false, ctx, err)) {
+        last_zone->deleteLater();
+        return false;
+      }
       // add to config
       ctx.config()->zones()->add(last_zone); ctx.add(last_zone, i);
     } else {
       // when extending the last zone, chop its name to remove the "... A" part.
       last_zone->setName(last_zonebasename);
+      if (! decodeZone(i, last_zone, true, ctx, err)) {
+        last_zone->deleteLater();
+        return false;
+      }
     }
   }
   return true;
 }
 
+bool
+D868UVCodeplug::decodeZone(int i, Zone *zone, bool isB, Context &ctx, const ErrorStack &err) {
+  Q_UNUSED(i); Q_UNUSED(zone); Q_UNUSED(isB); Q_UNUSED(ctx); Q_UNUSED(err)
+  return true;
+}
+
 bool
 D868UVCodeplug::linkZones(Context &ctx, const ErrorStack &err) {
   Q_UNUSED(err)
@@ -1191,10 +1227,18 @@ D868UVCodeplug::linkZones(Context &ctx, const ErrorStack &err) {
       else
         last_zone->A()->add(ctx.get<Channel>(cidx));
     }
+
+    if (! linkZone(i, last_zone, extend_last_zone, ctx, err))
+      return false;
   }
   return true;
 }
 
+bool
+D868UVCodeplug::linkZone(int i, Zone *zone, bool isB, Context &ctx, const ErrorStack &err) {
+  Q_UNUSED(i); Q_UNUSED(zone); Q_UNUSED(isB); Q_UNUSED(ctx); Q_UNUSED(err)
+  return true;
+}
 
 void
 D868UVCodeplug::allocateScanLists() {
@@ -1265,7 +1309,7 @@ D868UVCodeplug::linkScanLists(Context &ctx, const ErrorStack &err) {
     ScanListElement scanl(data(addr));
     // Create scanlist
     ScanList *obj = ctx.get<ScanList>(i);
-    // Link scanlists immediately, all channels are defined allready
+    // Link scanlists immediately, all channels are defined already
     ctx.config()->scanlists()->add(obj); scanl.linkScanListObj(obj, ctx);
   }
   return true;
@@ -1351,7 +1395,7 @@ D868UVCodeplug::createGPSSystems(Context &ctx, const ErrorStack &err) {
     uint16_t  bit = i%8, byte = i/8, bank = i/128, idx = i%128;
     if (0 == ((channel_bitmap[byte]>>bit) & 0x01))
       continue;
-    if (ctx.get<Channel>(i)->is<AnalogChannel>())
+    if (ctx.get<Channel>(i)->is<FMChannel>())
       continue;
     ChannelElement ch(data(CHANNEL_BANK_0 + bank*CHANNEL_BANK_OFFSET + idx*CHANNEL_SIZE));
     if (ch.txDigitalAPRS())
