@@ -264,13 +264,13 @@ RadioddityCodeplug::ChannelElement::enablePrivacy(bool enable) {
   setBit(0x00031, 4, enable);
 }
 
-DigitalChannel::TimeSlot
+DMRChannel::TimeSlot
 RadioddityCodeplug::ChannelElement::timeSlot() const {
-  return (getBit(0x0031, 6) ? DigitalChannel::TimeSlot::TS2 : DigitalChannel::TimeSlot::TS1);
+  return (getBit(0x0031, 6) ? DMRChannel::TimeSlot::TS2 : DMRChannel::TimeSlot::TS1);
 }
 void
-RadioddityCodeplug::ChannelElement::setTimeSlot(DigitalChannel::TimeSlot ts) {
-  setBit(0x0031, 6, DigitalChannel::TimeSlot::TS2 == ts);
+RadioddityCodeplug::ChannelElement::setTimeSlot(DMRChannel::TimeSlot ts) {
+  setBit(0x0031, 6, DMRChannel::TimeSlot::TS2 == ts);
 }
 
 bool
@@ -290,13 +290,13 @@ RadioddityCodeplug::ChannelElement::enableNonSTEFrequency(bool enable) {
   setBit(0x0032, 5, enable);
 }
 
-AnalogChannel::Bandwidth
+FMChannel::Bandwidth
 RadioddityCodeplug::ChannelElement::bandwidth() const {
-  return (getBit(0x0033, 1) ? AnalogChannel::Bandwidth::Wide : AnalogChannel::Bandwidth::Narrow);
+  return (getBit(0x0033, 1) ? FMChannel::Bandwidth::Wide : FMChannel::Bandwidth::Narrow);
 }
 void
-RadioddityCodeplug::ChannelElement::setBandwidth(AnalogChannel::Bandwidth bw) {
-  setBit(0x0033, 1, AnalogChannel::Bandwidth::Wide == bw);
+RadioddityCodeplug::ChannelElement::setBandwidth(FMChannel::Bandwidth bw) {
+  setBit(0x0033, 1, FMChannel::Bandwidth::Wide == bw);
 }
 
 bool
@@ -348,23 +348,23 @@ RadioddityCodeplug::ChannelElement::toChannelObj(Codeplug::Context &ctx) const {
   Q_UNUSED(ctx)
   Channel *ch = nullptr;
   if (MODE_ANALOG == mode()) {
-    AnalogChannel *ach = new AnalogChannel(); ch = ach;
+    FMChannel *ach = new FMChannel(); ch = ach;
     switch (admitCriterion()) {
-    case ADMIT_ALWAYS: ach->setAdmit(AnalogChannel::Admit::Always); break;
-    case ADMIT_CH_FREE: ach->setAdmit(AnalogChannel::Admit::Free); break;
-    default: ach->setAdmit(AnalogChannel::Admit::Always); break;
+    case ADMIT_ALWAYS: ach->setAdmit(FMChannel::Admit::Always); break;
+    case ADMIT_CH_FREE: ach->setAdmit(FMChannel::Admit::Free); break;
+    default: ach->setAdmit(FMChannel::Admit::Always); break;
     }
     ach->setBandwidth(bandwidth());
     ach->setRXTone(rxTone());
     ach->setTXTone(txTone());
     ach->setSquelchDefault(); // There is no per-channel squelch setting
   } else {
-    DigitalChannel *dch = new DigitalChannel(); ch = dch;
+    DMRChannel *dch = new DMRChannel(); ch = dch;
     switch (admitCriterion()) {
-    case ADMIT_ALWAYS: dch->setAdmit(DigitalChannel::Admit::Always); break;
-    case ADMIT_CH_FREE: dch->setAdmit(DigitalChannel::Admit::Free); break;
-    case ADMIT_COLOR: dch->setAdmit(DigitalChannel::Admit::ColorCode); break;
-    default: dch->setAdmit(DigitalChannel::Admit::Always); break;
+    case ADMIT_ALWAYS: dch->setAdmit(DMRChannel::Admit::Always); break;
+    case ADMIT_CH_FREE: dch->setAdmit(DMRChannel::Admit::Free); break;
+    case ADMIT_COLOR: dch->setAdmit(DMRChannel::Admit::ColorCode); break;
+    default: dch->setAdmit(DMRChannel::Admit::Always); break;
     }
     dch->setTimeSlot(timeSlot());
     dch->setColorCode(txColorCode());
@@ -389,14 +389,14 @@ bool
 RadioddityCodeplug::ChannelElement::linkChannelObj(Channel *c, Context &ctx) const {
   // Link common
   if (hasScanList() && ctx.has<ScanList>(scanListIndex()))
-    c->setScanListObj(ctx.get<ScanList>(scanListIndex()));
+    c->setScanList(ctx.get<ScanList>(scanListIndex()));
   // Link digital channel
-  if (c->is<DigitalChannel>()) {
-    DigitalChannel *dc = c->as<DigitalChannel>();
+  if (c->is<DMRChannel>()) {
+    DMRChannel *dc = c->as<DMRChannel>();
     if (hasGroupList() && ctx.has<RXGroupList>(groupListIndex()))
       dc->setGroupListObj(ctx.get<RXGroupList>(groupListIndex()));
-    if (hasContact() && ctx.has<DigitalContact>(contactIndex()))
-      dc->setTXContactObj(ctx.get<DigitalContact>(contactIndex()));
+    if (hasContact() && ctx.has<DMRContact>(contactIndex()))
+      dc->setTXContactObj(ctx.get<DMRContact>(contactIndex()));
   }
   return true;
 
@@ -423,28 +423,28 @@ RadioddityCodeplug::ChannelElement::fromChannelObj(const Channel *c, Context &ct
   bool channelVOXEnabled = (! (c->voxDisabled()||c->defaultVOX()));
   enableVOX(defaultVOXEnabled || channelVOXEnabled);
 
-  if (c->scanListObj())
-    setScanListIndex(ctx.index(c->scanListObj()));
+  if (c->scanList())
+    setScanListIndex(ctx.index(c->scanList()));
 
-  if (c->is<AnalogChannel>()) {
-    const AnalogChannel *ac = c->as<const AnalogChannel>();
+  if (c->is<FMChannel>()) {
+    const FMChannel *ac = c->as<const FMChannel>();
     setMode(MODE_ANALOG);
     switch (ac->admit()) {
-    case AnalogChannel::Admit::Always: setAdmitCriterion(ADMIT_ALWAYS); break;
-    case AnalogChannel::Admit::Free: setAdmitCriterion(ADMIT_CH_FREE); break;
+    case FMChannel::Admit::Always: setAdmitCriterion(ADMIT_ALWAYS); break;
+    case FMChannel::Admit::Free: setAdmitCriterion(ADMIT_CH_FREE); break;
     default: setAdmitCriterion(ADMIT_ALWAYS);
     }
     setBandwidth(ac->bandwidth());
     setRXTone(ac->rxTone());
     setTXTone(ac->txTone());
     // no per channel squelch setting
-  } else if (c->is<DigitalChannel>()) {
-    const DigitalChannel *dc = c->as<const DigitalChannel>();
+  } else if (c->is<DMRChannel>()) {
+    const DMRChannel *dc = c->as<const DMRChannel>();
     setMode(MODE_DIGITAL);
     switch (dc->admit()) {
-    case DigitalChannel::Admit::Always: setAdmitCriterion(ADMIT_ALWAYS); break;
-    case DigitalChannel::Admit::Free: setAdmitCriterion(ADMIT_CH_FREE); break;
-    case DigitalChannel::Admit::ColorCode: setAdmitCriterion(ADMIT_COLOR); break;
+    case DMRChannel::Admit::Always: setAdmitCriterion(ADMIT_ALWAYS); break;
+    case DMRChannel::Admit::Free: setAdmitCriterion(ADMIT_CH_FREE); break;
+    case DMRChannel::Admit::ColorCode: setAdmitCriterion(ADMIT_COLOR); break;
     }
     setTimeSlot(dc->timeSlot());
     setRXColorCode(dc->colorCode());
@@ -607,7 +607,7 @@ void
 RadioddityCodeplug::ContactElement::clear() {
   setName("");
   setNumber(0);
-  setType(DigitalContact::GroupCall);
+  setType(DMRContact::GroupCall);
   enableRing(0);
   setRingStyle(0);
   setUInt8(0x017, 0x00);
@@ -636,22 +636,22 @@ RadioddityCodeplug::ContactElement::setNumber(unsigned id) {
   setBCD8_be(0x0010, id);
 }
 
-DigitalContact::Type
+DMRContact::Type
 RadioddityCodeplug::ContactElement::type() const {
   switch (getUInt8(0x0014)) {
-  case 0: return DigitalContact::GroupCall;
-  case 1: return DigitalContact::PrivateCall;
-  case 2: return DigitalContact::AllCall;
+  case 0: return DMRContact::GroupCall;
+  case 1: return DMRContact::PrivateCall;
+  case 2: return DMRContact::AllCall;
   default: break;
   }
-  return DigitalContact::PrivateCall;
+  return DMRContact::PrivateCall;
 }
 void
-RadioddityCodeplug::ContactElement::setType(DigitalContact::Type type) {
+RadioddityCodeplug::ContactElement::setType(DMRContact::Type type) {
   switch (type) {
-  case DigitalContact::GroupCall: setUInt8(0x0014, 0); break;
-  case DigitalContact::PrivateCall: setUInt8(0x0014, 1); break;
-  case DigitalContact::AllCall: setUInt8(0x0014, 2); break;
+  case DMRContact::GroupCall: setUInt8(0x0014, 0); break;
+  case DMRContact::PrivateCall: setUInt8(0x0014, 1); break;
+  case DMRContact::AllCall: setUInt8(0x0014, 2); break;
   }
 }
 
@@ -677,16 +677,16 @@ RadioddityCodeplug::ContactElement::setRingStyle(unsigned style) {
   setUInt8(0x0016, style);
 }
 
-DigitalContact *
+DMRContact *
 RadioddityCodeplug::ContactElement::toContactObj(Context &ctx) const {
   Q_UNUSED(ctx)
   if (! isValid())
     return nullptr;
-  return new DigitalContact(type(), name(), number(), ring());
+  return new DMRContact(type(), name(), number(), ring());
 }
 
 void
-RadioddityCodeplug::ContactElement::fromContactObj(const DigitalContact *cont, Context &ctx) {
+RadioddityCodeplug::ContactElement::fromContactObj(const DMRContact *cont, Context &ctx) {
   Q_UNUSED(ctx)
   setName(cont->name());
   setNumber(cont->number());
@@ -976,8 +976,8 @@ RadioddityCodeplug::GroupListElement::toRXGroupListObj(Context &ctx) {
 bool
 RadioddityCodeplug::GroupListElement::linkRXGroupListObj(int ncnt, RXGroupList *lst, Context &ctx) const {
   for (int i=0; (i<16) && (i<ncnt); i++) {
-    if (ctx.has<DigitalContact>(member(i))) {
-      lst->addContact(ctx.get<DigitalContact>(member(i)));
+    if (ctx.has<DMRContact>(member(i))) {
+      lst->addContact(ctx.get<DMRContact>(member(i)));
     } else {
       logError() << "Cannot link group list '" << lst->name()
                 << "': Member index " << member(i) << " does not refer to a digital contact.";
@@ -990,10 +990,19 @@ RadioddityCodeplug::GroupListElement::linkRXGroupListObj(int ncnt, RXGroupList *
 void
 RadioddityCodeplug::GroupListElement::fromRXGroupListObj(const RXGroupList *lst, Context &ctx) {
   setName(lst->name());
+  int j = 0;
+  // Iterate over all 15 entries in the codeplug
   for (int i=0; i<15; i++) {
-    if (lst->count() > i) {
-      setMember(i, ctx.index(lst->contact(i)));
+    if (lst->count() > j) {
+      // Skip non-private-call entries
+      while((lst->count() > j) && (DMRContact::GroupCall != lst->contact(j)->type())) {
+        logWarn() << "Contact '" << lst->contact(i)->name() << "' in group list '" << lst->name()
+                  << "' is not a group call. Skip entry.";
+        j++;
+      }
+      setMember(i, ctx.index(lst->contact(j))); j++;
     } else {
+      // Clear entry.
       clearMember(i);
     }
   }
@@ -2438,6 +2447,130 @@ RadioddityCodeplug::MessageBankElement::appendMessage(const QString msg) {
 }
 
 
+/* ********************************************************************************************* *
+ * Implementation of RadioddityCodeplug::EncryptionElement
+ * ********************************************************************************************* */
+RadioddityCodeplug::EncryptionElement::EncryptionElement(uint8_t *ptr, size_t size)
+  : Codeplug::Element(ptr, size)
+{
+  // pass...
+}
+
+RadioddityCodeplug::EncryptionElement::EncryptionElement(uint8_t *ptr)
+  : Element(ptr, 0x88)
+{
+  // pass...
+}
+
+RadioddityCodeplug::EncryptionElement::~EncryptionElement() {
+  // pass...
+}
+
+void
+RadioddityCodeplug::EncryptionElement::clear() {
+  setPrivacyType(PrivacyType::None);
+  for (int i=0; i<16; i++) {
+    clearBasicKey(i);
+  }
+}
+
+RadioddityCodeplug::EncryptionElement::PrivacyType
+RadioddityCodeplug::EncryptionElement::privacyType() const {
+  return PrivacyType(getUInt8(0x0000));
+}
+void
+RadioddityCodeplug::EncryptionElement::setPrivacyType(PrivacyType type) {
+  setUInt8(0x0000, (uint8_t)type);
+}
+
+bool
+RadioddityCodeplug::EncryptionElement::isBasicKeySet(unsigned n) const {
+  if (n>=16)
+    return false;
+  unsigned byte=n/8, bit =n%8;
+  return getBit(0x0002+byte, bit);
+}
+QByteArray
+RadioddityCodeplug::EncryptionElement::basicKey(unsigned n) const {
+  if (n >= 16)
+    return QByteArray();
+  return QByteArray((const char *)_data+0x0008+0x08*n, 4);
+}
+void
+RadioddityCodeplug::EncryptionElement::setBasicKey(unsigned n, const QByteArray &key) {
+  if ((n>=16) || (4 != key.size()))
+    return;
+  unsigned byte=n/8, bit =n%8;
+  memcpy(_data+0x0008 + 0x08*n, key.data(), 4);
+  memcpy(_data+0x0008 + 0x08*n + 0x04, key.data(), 4);
+  setBit(0x0002+byte, bit);
+  setPrivacyType(PrivacyType::Basic);
+}
+void
+RadioddityCodeplug::EncryptionElement::clearBasicKey(unsigned n) {
+  if (n>=16)
+    return;
+  unsigned byte=n/8, bit =n%8;
+  memset(_data+0x0008 + 0x08*n, 0xff, 4);
+  memset(_data+0x0008 + 0x08*n + 0x04, 0xff, 4);
+  clearBit(0x0002+byte, bit);
+}
+
+bool
+RadioddityCodeplug::EncryptionElement::fromCommercialExt(CommercialExtension *ext, Context &ctx) {
+  clear();
+
+  if (ext->encryptionKeys()->count() > 16) {
+    logError() << "Cannot encode encryption extension. Can only encode 16 keys.";
+    return false;
+  }
+
+  for (int i=0; i<ext->encryptionKeys()->count(); i++) {
+    if (! ext->encryptionKeys()->get(i)->is<DMREncryptionKey>()) {
+      logError() << "Can only encode basic encryption keys.";
+      return false;
+    }
+    DMREncryptionKey *key = ext->encryptionKeys()->get(i)->as<DMREncryptionKey>();
+    if (key->key().size() != 4) {
+      logError() << "Can only encode 32bit basic encryption keys.";
+      return false;
+    }
+    setBasicKey(i, key->key());
+    ctx.add(key, i+1);
+  }
+
+  return true;
+}
+
+bool RadioddityCodeplug::EncryptionElement::updateCommercialExt(Context &ctx) {
+  if (PrivacyType::None == privacyType())
+    return false;
+
+  CommercialExtension *ext = ctx.config()->commercialExtension();
+  for (int i=0; i<16; i++) {
+    if (! isBasicKeySet(i))
+      continue;
+    // Assemble key
+    DMREncryptionKey *key = new DMREncryptionKey();
+    key->setName(QString("Basic Key %1").arg(i+1));
+    key->fromHex(basicKey(i).toHex());
+    // add key to extension
+    ext->encryptionKeys()->add(key);
+    // register key index
+    ctx.add(key, i+1);
+  }
+
+  return ext;
+}
+
+bool
+RadioddityCodeplug::EncryptionElement::linkCommercialExt(CommercialExtension *ext, Context &ctx) {
+  Q_UNUSED(ext); Q_UNUSED(ctx);
+  // Keys do not need any linking step
+  return true;
+}
+
+
 /* ********************************************************************************************* *
  * Implementation of RadioddityCodeplug
  * ********************************************************************************************* */
@@ -2457,7 +2590,7 @@ RadioddityCodeplug::clear() {
   clearGeneralSettings();
   // Clear button settings
   clearButtonSettings();
-  // Clear messsages
+  // Clear messages
   clearMessages();
   // Clear emergency systems
   //clearEmergencySystems();
@@ -2479,6 +2612,8 @@ RadioddityCodeplug::clear() {
   clearScanLists();
   // clear group lists
   clearGroupLists();
+  // clear encryption keys
+  clearEncryption();
 }
 
 bool
@@ -2492,8 +2627,8 @@ RadioddityCodeplug::index(Config *config, Context &ctx, const ErrorStack &err) c
 
   // Map digital and DTMF contacts
   for (int i=0, d=0, a=0; i<config->contacts()->count(); i++) {
-    if (config->contacts()->contact(i)->is<DigitalContact>()) {
-      ctx.add(config->contacts()->contact(i)->as<DigitalContact>(), d+1); d++;
+    if (config->contacts()->contact(i)->is<DMRContact>()) {
+      ctx.add(config->contacts()->contact(i)->as<DMRContact>(), d+1); d++;
     } else if (config->contacts()->contact(i)->is<DTMFContact>()) {
       ctx.add(config->contacts()->contact(i)->as<DTMFContact>(), a+1); a++;
     }
@@ -2591,6 +2726,11 @@ RadioddityCodeplug::encodeElements(const Flags &flags, Context &ctx, const Error
     return false;
   }
 
+  if (! this->encodeEncryption(ctx.config(), flags, ctx, err)) {
+    errMsg(err) << "Cannot encode encryption keys.";
+    return true;
+  }
+
   return true;
 }
 
@@ -2632,6 +2772,11 @@ RadioddityCodeplug::decodeElements(Context &ctx, const ErrorStack &err) {
     return false;
   }
 
+  if (! this->createEncryption(ctx.config(), ctx, err)) {
+    errMsg(err) << "Cannot decode encryption keys.";
+    return false;
+  }
+
   if (! this->createZones(ctx.config(), ctx, err)) {
     errMsg(err) << "Cannot create zones.";
     return false;
@@ -2667,5 +2812,10 @@ RadioddityCodeplug::decodeElements(Context &ctx, const ErrorStack &err) {
     return false;
   }
 
+  if (! this->linkEncryption(ctx.config(), ctx, err)) {
+    errMsg(err) << "Cannot link encryption keys.";
+    return false;
+  }
+
   return true;
 }
