@@ -2,12 +2,15 @@
 #include <QMainWindow>
 #include <QtUiTools>
 #include <QDesktopServices>
+#include <QTranslator>
+#include <QStandardPaths>
 
 #include "logger.hh"
 #include "radio.hh"
 #include "codeplug.hh"
 #include "config.h"
 #include "settings.hh"
+#include "radiolimits.hh"
 #include "verifydialog.hh"
 #include "analogchanneldialog.hh"
 #include "digitalchanneldialog.hh"
@@ -17,7 +20,7 @@
 #include "gpssystemdialog.hh"
 #include "roamingzonedialog.hh"
 #include "aprssystemdialog.hh"
-#include "repeaterdatabase.hh"
+#include "repeaterbookcompleter.hh"
 #include "userdatabase.hh"
 #include "talkgroupdatabase.hh"
 #include "searchpopup.hh"
@@ -37,10 +40,21 @@
 #include "deviceselectiondialog.hh"
 #include "radioselectiondialog.hh"
 
+inline QStringList getLanguages() {
+  QStringList languages = {QLocale::system().name()};
+  if (languages.last().contains("_")) {
+    languages.append(languages.last().split("_").first());
+  }
+  return languages;
+}
+
+inline QString getLocalePath(const QString &language) {
+  return QDir(LOCALE_DIRECTORY "/" + language + "/LC_MESSAGES/").absolutePath();
+}
 
 Application::Application(int &argc, char *argv[])
-  : QApplication(argc, argv), _config(nullptr), _mainWindow(nullptr), _repeater(nullptr),
-    _lastDevice()
+  : QApplication(argc, argv), _config(nullptr), _mainWindow(nullptr), _translator(nullptr),
+    _repeater(nullptr), _lastDevice()
 {
   setApplicationName("qdmr");
   setOrganizationName("DM3MAT");
@@ -50,12 +64,33 @@ Application::Application(int &argc, char *argv[])
   QString logdir = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation);
   Logger::get().addHandler(new FileLogHandler(logdir+"/qdmr.log"));
 
+  // register icon themes
+  QStringList iconPaths = QIcon::themeSearchPaths();
+  iconPaths.prepend(":/icons");
+  QIcon::setThemeSearchPaths(iconPaths);
+  onPaletteChanged(palette());
+
+  // handle translations
+  _translator = new QTranslator(this);
+  foreach (QString language, getLanguages()) {
+    logDebug() << "Search for translation in '" << getLocalePath(language) << "'.";
+    if (_translator->load("qdmr", getLocalePath(language), "", "_qt.qm")) {
+      this->installTranslator(_translator);
+      logDebug() << "Installed translator for locale '" << QLocale::system().name() << "'.";
+      break;
+    }
+  }
+
+  // load settings
   Settings settings;
-  _repeater   = new RepeaterDatabase(settings.position(), 7, this);
+  // load databases
+  _repeater   = new RepeaterBookList(this);
   _users      = new UserDatabase(30, this);
   _talkgroups = new TalkGroupDatabase(30, this);
-  _config = new Config(this);
+  // create empty codeplug
+  _config     = new Config(this);
 
+  // Handle args (if there are some)
   if (argc>1) {
     QFileInfo info(argv[1]);
     QFile file(argv[1]);
@@ -79,6 +114,7 @@ Application::Application(int &argc, char *argv[])
     }
   }
 
+  // load position
   _currentPosition = settings.position();
   _source = QGeoPositionInfoSource::createDefaultSource(this);
   if (_source) {    
@@ -103,6 +139,18 @@ Application::~Application() {
   _mainWindow = nullptr;
 }
 
+bool
+Application::isDarkMode() const {
+  return isDarkMode(palette());
+}
+
+bool
+Application::isDarkMode(const QPalette &palette) const {
+  int text_hsv_value = palette.color(QPalette::WindowText).value(),
+      bg_hsv_value = palette.color(QPalette::Background).value();
+  return text_hsv_value > bg_hsv_value;
+}
+
 
 QMainWindow *
 Application::mainWindow() {
@@ -122,8 +170,7 @@ Application::talkgroup() const {
   return _talkgroups;
 }
 
-RepeaterDatabase *
-Application::repeater() const{
+RepeaterBookList *Application::repeater() const{
   return _repeater;
 }
 
@@ -134,6 +181,7 @@ Application::createMainWindow() {
     return _mainWindow;
 
   Settings settings;
+  logDebug() << "Create main window using icon theme '" << QIcon::themeName() << "'.";
 
   QUiLoader loader;
   QFile uiFile("://ui/mainwindow.ui");
@@ -159,7 +207,6 @@ Application::createMainWindow() {
 
   QAction *refreshCallsignDB  = _mainWindow->findChild<QAction*>("actionRefreshCallsignDB");
   QAction *refreshTalkgroupDB  = _mainWindow->findChild<QAction*>("actionRefreshTalkgroupDB");
-  QAction *refreshRepeaterDB  = _mainWindow->findChild<QAction*>("actionRefreshRepeaterDB");
 
   QAction *about   = _mainWindow->findChild<QAction*>("actionAbout");
   QAction *sett    = _mainWindow->findChild<QAction*>("actionSettings");
@@ -176,7 +223,6 @@ Application::createMainWindow() {
 
   connect(refreshCallsignDB, SIGNAL(triggered()), _users, SLOT(download()));
   connect(refreshTalkgroupDB, SIGNAL(triggered()), _talkgroups, SLOT(download()));
-  connect(refreshRepeaterDB, SIGNAL(triggered()), _repeater, SLOT(download()));
 
   connect(findDev, SIGNAL(triggered()), this, SLOT(detectRadio()));
   connect(verCP, SIGNAL(triggered()), this, SLOT(verifyCodeplug()));
@@ -191,9 +237,12 @@ Application::createMainWindow() {
   tabs->addTab(_generalSettings, tr("Settings"));
   if (settings.showCommercialFeatures()) {
     _generalSettings->hideDMRID(true);
-    _generalSettings->hideExtensions(false);
   } else {
     _generalSettings->hideDMRID(false);
+  }
+  if (settings.showExtensions()) {
+    _generalSettings->hideExtensions(false);
+  } else {
     _generalSettings->hideExtensions(true);
   }
 
@@ -231,12 +280,14 @@ Application::createMainWindow() {
 
   // Wire-up "extension view"
   _extensionView = new ExtensionView();
-  _extensionView->setObject(_config);
+  _extensionView->setObject(_config, _config);
   tabs->addTab(_extensionView, tr("Extensions"));
 
   if (! settings.showCommercialFeatures()) {
     tabs->removeTab(tabs->indexOf(_radioIdTab));
     _radioIdTab->setHidden(true);
+  }
+  if (! settings.showExtensions()) {
     tabs->removeTab(tabs->indexOf(_extensionView));
     _extensionView->setHidden(true);
   }
@@ -332,7 +383,7 @@ Application::saveCodeplug() {
 
   if (filename.endsWith(".conf") || filename.endsWith("csv")){
     QMessageBox::critical(nullptr, tr("Please use new YAML format."),
-                          tr("Saveing in the old table-based conf format was disabled with 0.9.0. "
+                          tr("Saving in the old table-based conf format was disabled with 0.9.0. "
                              "Reading these files still works."));
     return;
   }
@@ -435,7 +486,7 @@ Application::detectRadio() {
 
 
 bool
-Application::verifyCodeplug(Radio *radio, bool showSuccess, const VerifyFlags &flags) {
+Application::verifyCodeplug(Radio *radio, bool showSuccess) {
   Radio *myRadio = radio;
 
   // If no radio is given -> try to detect the radio
@@ -444,13 +495,13 @@ Application::verifyCodeplug(Radio *radio, bool showSuccess, const VerifyFlags &f
   if (nullptr == myRadio)
     return false;
 
-
+  Settings settings;
+  RadioLimitContext ctx(settings.ignoreFrequencyLimits());
+  myRadio->limits().verifyConfig(_config, ctx);
   bool verified = true;
-  QList<VerifyIssue> issues;
-  VerifyIssue::Type maxIssue = myRadio->verifyConfig(_config, issues, flags);
-  if ( (flags.ignoreWarnings && (maxIssue>VerifyIssue::WARNING)) ||
-       ((!flags.ignoreWarnings) && (maxIssue>=VerifyIssue::WARNING)) ) {
-    VerifyDialog dialog(issues, (nullptr != radio));
+  if ( (settings.ignoreVerificationWarning() && (ctx.maxSeverity()>RadioLimitIssue::Warning)) ||
+       ((!settings.ignoreVerificationWarning()) && (ctx.maxSeverity()>=RadioLimitIssue::Warning)) ) {
+    VerifyDialog dialog(ctx, (nullptr != radio));
     if (QDialog::Accepted != dialog.exec())
       verified = false;
   } else if (showSuccess) {
@@ -541,15 +592,7 @@ Application::uploadCodeplug() {
   if (nullptr == radio)
     return;
 
-
-  // Verify codeplug against the detected radio before uploading,
-  // but do not show a message on success.
-  VerifyFlags flags = {
-    settings.ignoreVerificationWarning(),
-    settings.ignoreFrequencyLimits()
-  };
-
-  if (! verifyCodeplug(radio, false, flags)) {
+  if (! verifyCodeplug(radio, false)) {
     radio->deleteLater();
     return;
   }
@@ -580,7 +623,7 @@ Application::uploadCallsignDB() {
   if (nullptr == radio)
     return;
 
-  if (! radio->features().hasCallsignDB) {
+  if (! radio->limits().hasCallSignDB()) {
     logDebug() << "Radio " << radio->name() << " does not support call-sign DB.";
     QMessageBox::information(nullptr, tr("Cannot write call-sign DB."),
                              tr("The detected radio '%1' does not support "
@@ -589,7 +632,7 @@ Application::uploadCallsignDB() {
     radio->deleteLater();
     return;
   }
-  if (! radio->features().callsignDBImplemented) {
+  if (! radio->limits().callSignDBImplemented()) {
     logDebug() << "Radio " << radio->name()
                << " does support call-sign DB but it is not implemented yet.";
     QMessageBox::critical(nullptr, tr("Cannot write call-sign DB."),
@@ -668,7 +711,7 @@ Application::onCodeplugUploaded(Radio *radio) {
   _mainWindow->findChild<QProgressBar *>("progress")->setVisible(false);
   _mainWindow->setEnabled(true);
 
-  logError() << "Write complete.";
+  logDebug() << "Write complete.";
 
   if (radio->wait(250))
     radio->deleteLater();
@@ -696,22 +739,26 @@ Application::showSettings() {
         tabs->insertTab(tabs->indexOf(_generalSettings)+1, _radioIdTab, tr("Radio IDs"));
         _mainWindow->update();
       }
-      if (-1 == tabs->indexOf(_extensionView)) {
-        tabs->insertTab(tabs->indexOf(_roamingZoneList)+1, _extensionView, tr("Extensions"));
-        _mainWindow->update();
-      }
       _generalSettings->hideDMRID(true);
-      _generalSettings->hideExtensions(false);
     } else if (! settings.showCommercialFeatures()) {
       if (-1 != tabs->indexOf(_radioIdTab)) {
         tabs->removeTab(tabs->indexOf(_radioIdTab));
         _mainWindow->update();
       }
+      _generalSettings->hideDMRID(false);
+    }
+    // Handle extensions
+    if (settings.showExtensions()) {
+      if (-1 == tabs->indexOf(_extensionView)) {
+        tabs->insertTab(tabs->indexOf(_roamingZoneList)+1, _extensionView, tr("Extensions"));
+        _mainWindow->update();
+      }
+      _generalSettings->hideExtensions(false);
+    } else {
       if (-1 != tabs->indexOf(_extensionView)) {
         tabs->removeTab(tabs->indexOf(_extensionView));
         _mainWindow->update();
       }
-      _generalSettings->hideDMRID(false);
       _generalSettings->hideExtensions(true);
     }
   }
@@ -730,18 +777,18 @@ Application::showAbout() {
   radioTab->setColumnCount(1);
   QHash<QString, QTreeWidgetItem*> items;
   foreach (RadioInfo radio, RadioInfo::allRadios(false)) {
-    if (! items.contains(radio.manufactuer()))
-      items.insert(radio.manufactuer(),
-                   new QTreeWidgetItem(QStringList(radio.manufactuer())));
-    items[radio.manufactuer()]->addChild(
+    if (! items.contains(radio.manufacturer()))
+      items.insert(radio.manufacturer(),
+                   new QTreeWidgetItem(QStringList(radio.manufacturer())));
+    items[radio.manufacturer()]->addChild(
           new QTreeWidgetItem(QStringList(radio.name())));
     foreach (RadioInfo alias, radio.alias()) {
-      if (! items.contains(alias.manufactuer()))
-        items.insert(alias.manufactuer(),
-                     new QTreeWidgetItem(QStringList(alias.manufactuer())));
-      items[alias.manufactuer()]->addChild(
+      if (! items.contains(alias.manufacturer()))
+        items.insert(alias.manufacturer(),
+                     new QTreeWidgetItem(QStringList(alias.manufacturer())));
+      items[alias.manufacturer()]->addChild(
             new QTreeWidgetItem(QStringList(tr("%1 (alias for %2 %3)").arg(alias.name())
-                                            .arg(radio.manufactuer()).arg(radio.name()))));
+                                            .arg(radio.manufacturer()).arg(radio.name()))));
     }
   }
   radioTab->insertTopLevelItems(0, items.values());
@@ -783,5 +830,16 @@ Application::position() const {
   return _currentPosition;
 }
 
+void
+Application::onPaletteChanged(const QPalette &palette) {
+  // Set theme based on UI mode (light vs. dark).
+  if (isDarkMode(palette)) {
+    QIcon::setThemeName("dark");
+    logDebug() << "Set icon theme to 'dark'.";
+  } else {
+    QIcon::setThemeName("light");
+    logDebug() << "Set icon theme to 'light'.";
+  }
+}
 
 
